/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_GL_CYLINDERARRAY_H
#define X3DTK_GL_CYLINDERARRAY_H

#include "GL_SceneGraphTypes.h"

#include <vector>
#include <list>
#include <utility>

namespace X3DTK {
namespace GL {

/*! \brief Class declared in <b><X3DTK/GL/scenegraph.h></b> a singleton manager
 *  of the vertex arrays necessary to draw a triangulated cylinder. 
 *
 *  This class is used by Cylinder to avoid memory duplication and provides the arrays required by the OpenGL command
 *  drawElements().
 *  
 *  \ingroup GLSceneGraph
 */
 
class CylinderDrawArray
{
public:
  /// Gets the CylinderDrawArray of section section.
  static CylinderDrawArray *getInstanceOfSection(unsigned int section);
  /// Removes the CylinderDrawArray of section section.
  void removeInstance();
  
  /// Gets the size of the bottom.
  unsigned int getCylinderBottomSize() const;
  /// Gets the size of the side.
  unsigned int getCylinderSideSize() const;
  /// Gets the size of the top.
  unsigned int getCylinderTopSize() const;
  /// Gets the address of the bottom vertex array.
  const void *getCylinderBottomVertexArrayAddress() const;
  /// Gets the address of the side vertex array.
  const void *getCylinderSideVertexArrayAddress() const;
  /// Gets the address of the top vertex array.
  const void *getCylinderTopVertexArrayAddress() const;
  /// Gets the address of the bottom index array.
  const unsigned int *getCylinderBottomIndexArrayAddress() const;
  /// Gets the address of the side index array.
  const unsigned int *getCylinderSideIndexArrayAddress() const;
  /// Gets the address of the top index array. 
  const unsigned int *getCylinderTopIndexArrayAddress() const;

private:
  unsigned int _section;
  
  CylinderDrawArray(unsigned int section); 

  typedef struct 
  {
    unsigned int count;
    CylinderDrawArray *ref;
  } data;
  
  typedef std::list<std::pair<unsigned int, data> > refList;

  static refList _refList;
  
  std::vector<N3F_V3F> _cylinderBottomVertexArray;
  std::vector<N3F_V3F> _cylinderSideVertexArray;
  std::vector<N3F_V3F>_cylinderTopVertexArray;
  std::vector<unsigned int> _cylinderBottomIndexArray;
  std::vector<unsigned int> _cylinderSideIndexArray;
  std::vector<unsigned int> _cylinderTopIndexArray;
};

}
}

#endif
