/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_GL_MATERIAL_H
#define X3DTK_GL_MATERIAL_H

#include "GL_X3DMaterialNode.h"

#include <vector>

namespace X3DTK {
namespace GL {

/*! \brief Class declared in <b><X3DTK/GL/scenegraph.h></b> providing the GL implementation
 *  of X3D::Material.
 *
 *  \ingroup GLSceneGraph
 */
 
class Material : public X3DMaterialNode
{
public:
  /// Constructor.
  Material();
  /// Destructor.
  virtual ~Material();
  
  /// Sets the diffuse color attribute.
  void setDiffuseColor(const MFFloat &diffuseColor);
  /// Sets the ambient color attribute.
  void setAmbientColor(const MFFloat &ambientColor);
  /// Sets the emissisve color attribute.
  void setEmissiveColor(const MFFloat &emissiveColor);
  /// Sets the shininess attribute.
  void setShininess(const SFFloat &shininess);
  /// Sets the specular color attribute.
  void setSpecularColor(const MFFloat &specularColor);

  /// Gets the diffuse color attribute.
  inline const MFFloat &getDiffuseColor() const {return _diffuseColor;};
  /// Gets the ambient color attribute.
  inline const MFFloat &getAmbientColor() const {return _ambientColor;};
  /// Gets the emissisve color attribute.
  inline const MFFloat &getEmissiveColor() const {return _emissiveColor;};
  /// Gets the shininess attribute.
  inline const SFFloat &getShininess() const {return _shininess;};
  /// Gets the specular color attribute.
  inline const MFFloat &getSpecularColor() const {return _specularColor;};

  /// Updates the attributes of the node.
  virtual void update();

private:  
  /// Diffuse color attribute.
  MFFloat _diffuseColor;
  /// Ambient color attribute.
  MFFloat _ambientColor;
  /// Emissisve color attribute.
  MFFloat _emissiveColor;
  /// Shininess attribute.
  SFFloat _shininess;
  /// Specular color attribute.
  MFFloat _specularColor;
};

}
}

#endif
