/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_GL_X3DCOMPOSEDGEOMETRYNODE_H
#define X3DTK_GL_X3DCOMPOSEDGEOMETRYNODE_H

#include "GL_X3DGeometryNode.h"

namespace X3DTK {
namespace GL {

/*! \brief Base class declared in <b><X3DTK/GL/scenegraph.h></b> providing the GL implementation
 *  of X3D::X3DComposedGeometryNode.
 *
 *  \ingroup GLSceneGraph
 */
 
class X3DComposedGeometryNode : public X3DGeometryNode
{
public:
  /// Constructor.
  X3DComposedGeometryNode();
  /// Destructor.
  virtual ~X3DComposedGeometryNode();

  /// Sets the ccw attribute.
  void setCcw(const SFBool &ccw);
  /// Sets the solid attribute.
  void setSolid(const SFBool &solid);
  /// Sets the color attribute.
  void setColor(const SFBool &color);
  /// Sets the texCoord attribute.
  void setTexCoord(const SFBool &texCoord);
  /// Sets the normalPerVertex attribute.  
  void setNormalPerVertex(const SFBool &normalPerVertex);
  /// Sets the vertex format.
  void setVertexFormat(const GLenum &format);
  
  /// gets the ccw attribute.
  inline const SFBool &getCcw() const {return _ccw;};
  /// Gets the solid attribute.
  inline const SFBool &getSolid() const {return _solid;};
  /// Gets the color attribute.
  inline const SFBool &getColor() const {return _color;};
  /// Gets the texCoord attribute.
  inline const SFBool &getTexCoord() const {return _texCoord;};
  /// Gets the normalPerVertex attribute.  
  inline const SFBool &getNormalPerVertex() {return _normalPerVertex;};
  /// Gets the vertex format.
  inline const GLenum &getVertexFormat() const {return _format;};

private:  
  /// Ccw attribute.
  SFBool _ccw;
  /// Solid attribute.
  SFBool _solid;
  /// Color attribute.
  SFBool _color;
  /// TexCoord attribute.
  SFBool _texCoord;
  /// NormalPerVertex.
  SFBool _normalPerVertex;
  /// vertex format.
  GLenum _format;
};

}
}

#endif
