/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_MESH_CONNECTEDCOMCOMPUTER_H
#define X3DTK_MESH_CONNECTEDCOMCOMPUTER_H

#include "X3DMultiPassProcessor.h"
#include "MESH_SceneGraphTypes.h"
#include "MESH_Mesh.h"
#include "MESH_Scene.h"
#include "MESH_MeshData.h"

namespace X3DTK {

class Progress;

namespace MESH {

/*! \brief Class declared in <b><X3DTK/MESH/connectedcomcomputer.h></b> providing a processor that 
 *  computes the connected components of the MESH scene graph.
 *
 *  The connected components are implicitly defined by the X3DTK::MESH::Vertex nodes.
 *  A connected component is a set of vertices defined by the X3DTK::MESH::Vertex node
 *  and a set of faces which are the faces of different X3DTK::MESH::Mesh nodes parents
 *  of the X3DTK::MESH::Vertex node. Moreover, a connected component can be made of 
 *  several X3DTK::MESH::Mesh that can be not connected. The processor cuts these 
 *  X3DTK::MESH::Mesh into several connected X3DTK::MESH::Mesh nodes.
 *
 *  \ingroup MESH
 */

class ConnectedComComputer : public X3DMultiPassProcessor
{
public:
  /// Constructor.
  ConnectedComComputer();
  /// Destructor.
  virtual ~ConnectedComComputer();
  
  /// Computes the connected components of the scene.
  void compute(Scene *S);
  /// Computes the connected components.
  void compute(X3DNode *N);
  /// Sets the selected nodes to draw. If empty, then all the graph is drawn.
  void setSelectedNodes(const MFNode &selectedNodes);

  /// Progress variables.
  Progress *progress;
};

}
}

#endif

