/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_MESH_NORMALSINVERTER_H
#define X3DTK_MESH_NORMALSINVERTER_H

#include "MESH_NormalsInverterStateVariables.h"
#include "X3DOnePassProcessor.h"
#include "MESH_SceneGraphTypes.h"
#include "DFSGraphTraversal.h"
#include "MESH_NormalsInverterCoreVisitor.h"
#include "MESH_Mesh.h"
#include "MESH_Scene.h"
#include "MESH_MeshData.h"

namespace X3DTK {
namespace MESH {

/*! \brief Class declared in <b><X3DTK/MESH/normalinverter.h></b> providing a processor that 
 *  inverts the faces of the MESH scene graph.
 *
 *  For each X3DTK::MESH::TemplateMesh, the faces are reversed and the normals inverted.
 *
 *  \ingroup MESH
 */

template<class MData, class VData, class EData, class FData, bool RW>
class TemplateNormalsInverter : public X3DOnePassProcessor
{
public:
  /// Constructor.
  TemplateNormalsInverter();
  /// Destructor.
  virtual ~TemplateNormalsInverter();

  /// Inverts the normals of the scene.
  void invert(Scene *S);
  /// Inverts the normals of the sub scene graph.
  void invert(X3DNode *N);
  /// Sets the selected nodes to draw. If empty, then all the graph is drawn.
  void setSelectedNodes(const MFNode &selectedNodes);
};

/// Default TemplateNormalsInverter.
typedef TemplateNormalsInverter<MeshData, VertexData, EdgeData, FaceData, true> NormalsInverter;

}
}

#include "MESH_NormalsInverter.inl"

#endif

