/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_MESH_NORMALSINVERTERSTATEVARIABLES_H
#define X3DTK_MESH_NORMALSINVERTERSTATEVARIABLES_H

#include "MESH_SceneGraphTypes.h"
#include "MESH_Mesh.h"

#include <list>
#include <vector>

namespace X3DTK {
namespace MESH {

/*! \brief Class declared in <b><X3DTK/MESH/TemplateNormalsInverter.h></b> providing the StateVariables
 *  of the MESH::TemplateNormalsInverter processor.
 *
 *  \ingroup MESH
 */

template<class MData, class VData, class EData, class FData, bool RW>
class TemplateNormalsInverterStateVariables : public StateVariables
{
public:
  /// Constructor.
  TemplateNormalsInverterStateVariables();

  /// Initializes the traversal.
  void init();
  /// Finishes the traversal.
  void finish();
  
  /// Adds the Coordinate.
  void addNode(SFNode N);
  /// Returns true if Coordinate has already been visited.
  bool getNode(SFNode N) const;
  /// Sets the selected nodes to draw. If empty, then all the graph is drawn.
  void setSelectedNodes(const MFNode &selectedNodes);
  /// Gets the selected nodes to draw. If empty, then all the graph is drawn.
  MFNode getSelectedNodes() const;
  
private:  
  std::list<SFNode> _vertexSetList;
  MFNode _selectedNodes;
};

}
}

#include "MESH_NormalsInverterStateVariables.inl"

#endif
