/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_MESH_TRANSFORM_H
#define X3DTK_MESH_TRANSFORM_H

#include "MESH_X3DGroupingNode.h"

namespace X3DTK {
namespace MESH {

/*! \brief Class declared in <b><X3DTK/MESH/scenegraph.h></b> defining a Transform,
 *  implementation of the X3D::Transform for the MESH scene graph.
 *
 *  \ingroup MeshSceneGraph
 */

class Transform : public X3DGroupingNode
{
public:
  /// Constructor.
  Transform();
  /// Virtual destructor.
  virtual ~Transform();
  
  /// Sets the center of the transformation.
  void setCenter(const SFVec3f &center);
  /// Sets the rotation of the transformation.
  void setRotation(const SFRotation &rotation);
  /// Sets the scale of the transformation.
  void setScale(const SFVec3f &scale);
  /// Sets the scale orientation of the transformation.
  void setScaleOrientation(const SFRotation &scaleOrientation);
  /// Sets the translation of the transformation.
  void setTranslation(const SFVec3f &translation);

  /// Gets the center of the transformation.
  inline const SFVec3f &getCenter() const {return _center;};
  /// Gets the rotation of the transformation.
  inline const SFRotation &getRotation() const {return _rotation;};
  /// Gets the scale of the transformation.
  inline const SFVec3f &getScale() const {return _scale;};
  /// Gets the scale orientation of the transformation.
  inline const SFRotation &getScaleOrientation() const {return _scaleOrientation;};
  /// Gets the translation of the transformation.
  inline const SFVec3f &getTranslation() const {return _translation;};
  
  /// Gets the transformation.
  SFMatrix34f getTransform() const;
  
private:
  SFVec3f _center;
  SFRotation _rotation;
  SFVec3f _scale;
  SFRotation _scaleOrientation;
  SFVec3f _translation;
};

}
}

#endif
