/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_PROCESSHISTORY_H
#define X3DTK_PROCESSHISTORY_H

#include "X3DBaseTypes.h"

#include <time.h>
#include <map>

namespace X3DTK {
namespace X3D {

class Scene;  

}

/*! \brief Class declared in <b><X3DTK/kernel.h></b> defining a container for process
 *  history of the scenes.
 *
 *  \ingroup processor
 */

class ProcessHistory
{
public:
  /// Constructor.
  ProcessHistory();
  /// Copy constructor.
  ProcessHistory(const ProcessHistory &H);

  /// Adds an entry at date.
  void addEntry(const SFString &content, const struct tm &time);
  /// Adds an entry.
  void addEntry(const SFString &content);
  /// Gets the process map.  
  inline std::list<std::pair<SFString, SFString> > getProcessMap() const {return _processMap;};
  
  /// Function initializing the time.
  static struct tm loadTime(const SFString &date);
  /// Function loading the Scene.
  static void loadScene(X3D::Scene *S, const ProcessHistory &history);

private:
  std::list<std::pair<SFString, SFString> > _processMap;
};

}

#endif
