/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_GL_SPHEREARRAY_H
#define X3DTK_GL_SPHEREARRAY_H

#include "GL_SceneGraphTypes.h"

#include <vector>

namespace X3DTK {
namespace GL {

/*! \brief Class declared in <b><X3DTK/GL/scenegraph.h></b> providing a singleton manager
 *  of the vertex arrays necessary to draw a triangulated sphere. 
 *  
 *  This class is used by Sphere to avoid memory duplication and provides the arrays required by the OpenGL command
 *  drawElements().
 *
 *  \ingroup GLSceneGraph
 */

class SphereDrawArray
{
public:
  /// Gets the CylinderDrawArray of section section.
  static SphereDrawArray *getInstanceOfSection(unsigned int section);
  /// Removes the CylinderDrawArray of section section.
  void removeInstance();
  
  /// Gets the number of vertices of the sphere.
  unsigned int getSphereSize() const;
  /// Gets the address of the first element of the vertex array.
  const void *getSphereVertexArrayAddress() const;
  /// Gets the address of the first index of the index array. 
  const unsigned int *getSphereIndexArrayAddress() const;
  
private:
  unsigned int _section;

  /// Constructs the arrays for a sphere of sub_theta subdivisions for the theta angle 
  /// and sub_phi subdivisions for the phi angle
  SphereDrawArray(unsigned int section);

  typedef struct 
  {
    unsigned int count;
    SphereDrawArray *ref;
  } data;
  
  typedef std::list<std::pair<unsigned int, data> > refList;

  static refList _refList;

  std::vector<N3F_V3F> _sphereVertexArray;
  std::vector<unsigned int> _sphereIndexArray;
};

}
}

#endif
