/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3D_BBOXUPDATERSTATEVARIABLES_H
#define X3DTK_X3D_BBOXUPDATERSTATEVARIABLES_H

#include "X3D_SceneGraphTypes.h"
#include "StateVariables.h"
#include "BBox.h"

#include <list>

namespace X3DTK {
namespace X3D {

/*! \brief Class declared in <b><X3DTK/X3D/bboxupdater.h></b> providing StateVariables
 *  for the X3D::BBoxUpdater processor.
 *
 *  \ingroup X3D
 */

class BBoxUpdaterStateVariables : public StateVariables
{
public:
  /// Constructor.
  BBoxUpdaterStateVariables();
  /// Destructor.
  virtual ~BBoxUpdaterStateVariables();
  
  /// Finishes the traversal.
  virtual void finish();
  
  /// Sets the staticProcessing.
  void setStaticProcessing(bool value);
  /// Adds the Bounding Box BB.
  void addBBox(SFNode N, BBox *BB);
  /// Sets the current BBox (for the Geometry nodes of Shape).
  void setShapeBBox(const BBox &BB);
  /// Adds the bounding Box to the list.
  void addBBoxToMergeList(const BBox &BB);
  
  /// Gets the staticProcessing value.
  inline bool getStaticProcessing() const {return _staticProcessing;};
  /// Gets the current BBox.
  BBox getShapeBBox() const {return _shapeBBox;};
  /// Gets the BBox of N.
  BBox *getBBox(SFNode N) const;
  /// Merges the BBox of the list.
  BBox mergeBBox();
  
private:
  bool _staticProcessing;
  BBox _shapeBBox;
  std::list<std::pair<SFNode, BBox *> > _BBList;
  std::list<BBox> _BBToMergeList;  
};

}
}

#endif
