/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3D_MESHBUILDER_H
#define X3DTK_X3D_MESHBUILDER_H

#include "X3DOnePassProcessor.h"
#include "X3D_MeshBuilderStateVariables.h"
#include "DFSGraphTraversal.h"
#include "X3D_MeshBuilderCoreVisitor.h"
#include "X3D_MeshBuilderGeometry3DVisitor.h"
#include "X3D_MeshBuilderGroupingVisitor.h"
#include "X3D_MeshBuilderShapeVisitor.h"
#include "X3D_MeshBuilderRenderingVisitor.h"
#include "X3D_Scene.h"
#include "MESH_Scene.h"
#include "MESH_MeshData.h"

namespace X3DTK {
namespace X3D {

/*! \brief Class declared in <b><X3DTK/X3D/meshbuilder.h></b> providing a processor
 *  that builds a MESH scene graph from the X3D scene graph.
 *
 *  Unlike X3DTK::X3D::GLBuilder, no information is lost during the conversion. There
 *  are less MESH nodes than X3D ones, but the processor clones the sub graphs which
 *  cannot be translated. For more details about the MESH scene graph, see the 
 * \ref MESHSceneGraphAPI "MESH" scene graph page.
 * 
 *  \ingroup X3D
 */
 
template<class MData, class VData, class EData, class FData, bool RW>
class TemplateMeshBuilder : public X3DOnePassProcessor
{     
public:
  /// Constructor. 
  TemplateMeshBuilder();
  /// Virtual destructor.
  virtual ~TemplateMeshBuilder();
    
  /// Builds the MESH sub scene graph from N.
  virtual MESH::X3DNode *build(X3DNode *N);
  
  /// Builds the MESH scene from S.
  MESH::Scene *build(Scene *S);
};

/// Default TemplateMeshBuilder.
typedef TemplateMeshBuilder<MESH::MeshData, MESH::VertexData, MESH::EdgeData, MESH::FaceData, true> MeshBuilder;

}
}

#include "X3D_MeshBuilder.inl"

#endif
