/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3D_SHAREDCOORDCOMPUTER_H
#define X3DTK_X3D_SHAREDCOORDCOMPUTER_H

#include "X3D_SceneGraphTypes.h"
#include "X3D_SharedCoordComputerStateVariables.h"
#include "X3DOnePassProcessor.h"

namespace X3DTK {
namespace X3D {

class Scene;

/*! \brief Class declared in <b><X3DTK/X3D/sharedcoordcomputer.h></b> providing a processor
 *  that computes the X3DTK::X3D::Coordinate nodes so that they are well-shared. 
 *  
 *  When an X3DTK::X3D::Coordinate node is shared by several shapes located in different 
 *  coordinate systems, the X3DTK::X3D::Coordinate node is duplicated, because it makes 
 *  no sense to share coordinates values that are not multiple vertices.
 *
 *  \ingroup X3D
 */

class SharedCoordComputer : public X3DOnePassProcessor
{
public:
  /// Constructor.
  SharedCoordComputer();
  /// Destructor.
  virtual ~SharedCoordComputer();
  
  /// Validates the file.
  virtual void compute(Scene *S);
  /// Validates the file.
  virtual void compute(X3DNode *N);
};

}
}

#endif
