/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3D_X3DBOUNDEDOBJECT_H
#define X3DTK_X3D_X3DBOUNDEDOBJECT_H

#include "X3DTypes.h"
#include "X3D_X3DObject.h"
#include "X3DFileElement.h"

namespace X3DTK {
namespace X3D {

/*! \brief Base class declared in <b><X3DTK/X3D/scenegraph.h></b>, part of the <a href="../../doc/X3DSpecs/Part01/components/group.html">
 *  Grouping component</a> defining an X3DBoundedObject
 *  (<a href="../../doc/X3DSpecs/Part01/components/group.html#X3DBoundedObject">
 *  documentation</a>).
 *
 *  \ingroup X3DSceneGraph
 */

class X3DBoundedObject : public X3DObject
{
public:
  /// Constructor.
  X3DBoundedObject();
  /// Constructs an X3DBoundedObject from its attributes.
  X3DBoundedObject(const SFVec3f &bboxCenter,
                   const SFVec3f &bboxSize);
  /// Virtual destructor.
  virtual ~X3DBoundedObject();

  /// Sets the BBoxCenter attribute.
  void setBBoxCenter(const SFVec3f &bboxCenter);
  /// Sets the BBoxSize attribute.
  void setBBoxSize(const SFVec3f &bboxSize);

  /// Gets the BBoxCenter attribute.
  inline const SFVec3f &getBBoxCenter() const {return _bboxCenter;};
  /// Gets the BBoxSize attribute.
  inline const SFVec3f &getBBoxSize() const {return _bboxSize;};

protected:
  /// Copy constructor.
  X3DBoundedObject(const X3DBoundedObject &N);

private:
  /// Center of the Bounding Box.
  SFVec3f _bboxCenter;
  /// Size of the Bounding Box.
  SFVec3f _bboxSize; 
};

}
}

#endif
